/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 * 
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */

package org.objectweb.medor.expression.lib;

import org.objectweb.jorm.type.api.PType;
import org.objectweb.jorm.type.api.PTypeSpace;
import org.objectweb.medor.expression.api.Comparator;
import org.objectweb.medor.expression.api.ExpressionException;
import org.objectweb.medor.expression.api.MalformedExpressionException;
import org.objectweb.medor.expression.api.Operand;
import org.objectweb.medor.expression.api.ParameterOperand;
import org.objectweb.medor.expression.api.TypingException;
import org.objectweb.medor.expression.api.Expression;
import org.objectweb.medor.expression.type.ExpressionTypeHelper;

/**
 *
 * @author Sebastien Chassande-Barrioz
 */
public abstract class BasicComparator
        extends BasicBinaryOperator
        implements Comparator {

	public BasicComparator() {
		super(PTypeSpace.BOOLEAN);
	}

	public BasicComparator(Expression e0, Expression e1) {
		super(PTypeSpace.BOOLEAN, e0, e1);
	}
	
    // IMPLEMENTATION OF THE Expression INTERFACE //
    //--------------------------------------------//

    /**
     * We supose that the comparisons can be performed between all types.
     */
    public org.objectweb.medor.expression.api.Operand
			evaluate(ParameterOperand[] pos, Object o)
            throws ExpressionException {
        Operand leftResult = expressions[0].evaluate(pos, o);
        Operand rightResult = expressions[1].evaluate(pos, o);
        PType rtype = ExpressionTypeHelper.getResultType(expressions[0].getType(),
                expressions[1].getType());
        switch (rtype.getTypeCode()) {
		case PType.TYPECODE_BOOLEAN:
			result.setValue(evaluate(leftResult.getBoolean(),
				rightResult.getBoolean()));
			break;
		case PType.TYPECODE_BYTE:
			result.setValue(evaluate(leftResult.getByte(),
				rightResult.getByte()));
			break;
		case PType.TYPECODE_CHAR:
			result.setValue(evaluate(leftResult.getChar(),
				rightResult.getChar()));
			break;
		case PType.TYPECODE_SHORT:
			result.setValue(evaluate(leftResult.getShort(),
				rightResult.getShort()));
			break;
		case PType.TYPECODE_INT:
			result.setValue(evaluate(leftResult.getInt(),
				rightResult.getInt()));
			break;
		case PType.TYPECODE_LONG:
			result.setValue(evaluate(leftResult.getLong(),
				rightResult.getLong()));
			break;
		case PType.TYPECODE_FLOAT:
			result.setValue(evaluate(leftResult.getFloat(),
				rightResult.getFloat()));
			break;
		case PType.TYPECODE_DOUBLE:
			result.setValue(evaluate(leftResult.getDouble(),
				rightResult.getDouble()));
			break;
		case PType.TYPECODE_STRING:
			result.setValue(evaluate(leftResult.getString(),
				rightResult.getString()));
			break;
		case PType.TYPECODE_OBJBOOLEAN:
		case PType.TYPECODE_OBJBYTE:
		case PType.TYPECODE_OBJCHAR:
		case PType.TYPECODE_OBJSHORT:
		case PType.TYPECODE_OBJINT:
		case PType.TYPECODE_OBJLONG:
		case PType.TYPECODE_OBJFLOAT:
		case PType.TYPECODE_OBJDOUBLE:
		case PType.TYPECODE_BIGDECIMAL:
		case PType.TYPECODE_BIGINTEGER:
			result.setValue(evaluate(leftResult.getObject(),
				rightResult.getObject()));
			break;
		default:
			throw new TypingException("Unmanaged the type: ("
				+ " typeCode: " + rtype.getTypeCode()
				+ " jormName: " + rtype.getJormName()
				+ " javaName: " + rtype.getJavaName()
				+ ") this=" + this
			);
        }
		return result;
    }


// IMPLEMENTATION OF THE Comparator INTERFACE //
//--------------------------------------------//

    public abstract boolean evaluate(Object op1, Object op2);

    public abstract boolean evaluate(int op1, int op2);

    public abstract boolean evaluate(int op1, short op2);

    public abstract boolean evaluate(int op1, long op2);

    public abstract boolean evaluate(int op1, float op2);

    public abstract boolean evaluate(int op1, double op2);

    public abstract boolean evaluate(float op1, float op2);

    public abstract boolean evaluate(float op1, short op2);

    public abstract boolean evaluate(float op1, int op2);

    public abstract boolean evaluate(float op1, long op2);

    public abstract boolean evaluate(float op1, double op2);

    public abstract boolean evaluate(char op1, char op2);

    public abstract boolean evaluate(long op1, long op2);

    public abstract boolean evaluate(long op1, float op2);

    public abstract boolean evaluate(long op1, double op2);

    public abstract boolean evaluate(double op1, double op2);

    public abstract boolean evaluate(double op1, short op2);

    public abstract boolean evaluate(double op1, int op2);

    public abstract boolean evaluate(double op1, float op2);

    public abstract boolean evaluate(double op1, long op2);

    public abstract boolean evaluate(String op1, String op2);

    public abstract boolean evaluate(boolean op1, boolean op2);

    public Operand compileExpression()
            throws ExpressionException, MalformedExpressionException {
        if ((expressions[0] != null) && (expressions[1] != null)) {
            expressions[0].compileExpression();
            expressions[1].compileExpression();
            if (!(expressions[0].getType() == null || expressions[1].getType() == null)) {
                if ((ExpressionTypeHelper.isArithmeticType(expressions[0].getType()) &&
                        (ExpressionTypeHelper.isArithmeticType(expressions[1].getType())))
                        //Equal operations OK between arithmetic types
                        ||
                        (expressions[0].getType().getTypeCode() ==
                        (expressions[1].getType().getTypeCode()))) {
                    //between operands which have the same types
                    if (!(expressions[0].getType().isa(expressions[1].getType())) &&
                            !(expressions[1].getType().isa(expressions[0].getType()))) {
                        throw new TypingException("incompatible types");
                    } else {
                        result = new BasicVariableOperand(PTypeSpace.BOOLEAN);
                        verified = true;
                    }
                } else
                // it is not a numeric type
                    throw new TypingException("Attempt an aritmetic type on a non arithmetic type");
            } else
            //syntax error
                throw new MalformedExpressionException("Null children value in Expression");
        } else
        // Operands not yet assigned
            throw new MalformedExpressionException("Null children value in Expression");
		return result;
    }
}
